#region Namespace references
/// Microsoft .NET Framework namespaces
using System;
using System.Data;
/// Microsoft CMS 2002 namespaces
using Microsoft.ContentManagement.Publishing;
#endregion

namespace ContentManagement.Extensions
{
	/// <summary>
	/// MCMS custom property database extension.
	/// </summary>
	/// <remarks>
	///		<para>
	///		MCMS Publishing API (PAPI) implements custom properties within the MCMS database.  The MCMS 
	///		PAPI implementation is somewhat limiting in that searching incurs a significant performance 
	///		hit and you can't create custom properties on the fly... it must be done only through 
	///		MCMS Site Manager.  Custom properties are also limited to text only values.
	///		</para>
	///		<para>
	///		The <b>CustomPropertyDbEx</b> moves all custom properties to a database and provides robust 
	///		searching and custom property management.
	///		</para>
	/// </remarks>
	public class CustomPropertyDbEx
	{
		#region Constants
		#endregion

		#region Objects
		// flag indicating this property has been marked for deletion
		private bool					m_deletePropertyFlag		=	false;
		// flag indicating this property exists in the database
		private bool					m_existingPropertyFlag	= false;

		// data access layer object
		private DataHandler		m_dataHandler;

		// properties of the custom prop
		private	Guid													m_uniqueID;
		private CustomPropertyDbExAssetType		m_assetType;
		private Guid													m_assetID;
		private string												m_cpKey				= string.Empty;
		private string												m_cpValue			= string.Empty;
		#endregion

		#region Constructor & Descructor
		/// <overloads>
		/// Initializes a new instance of the <b>CustomPropertyDbEx</b> class.
		/// </overloads>
		/// <summary>
		/// Creates a new <b>CustomPropertyDbEx</b> and sets the <see cref="AssetGuid"/> and 
		/// <see cref="AssetType"/> for the provided <paramref>channel</paramref>.
		/// </summary>
		/// <param name="channel">MCMS channel.</param>
		/// <example>
		/// <code>
		/// public void CreateProperties(Channel channel)
		/// {
		///   CustomPropertyDbEx cp = new CustomPropertyDbEx(channel);
		///   cp.Key = "TimeZone";
		///   cp.Value = "EST";
		///   cp.Commit();
		/// }
		/// </code>
		/// </example>
		public CustomPropertyDbEx(Channel channel)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= CustomPropertyDbExAssetType.Channel;
			this.m_assetID		= new Guid(channel.Guid);
		}

		
		/// <summary>
		/// Creates a new <b>CustomPropertyDbEx</b> and sets the <see cref="AssetGuid"/> and 
		/// <see cref="AssetType"/> for the provided <paramref>channel</paramref> as well as 
		/// setting the <see cref="Key"/> and <see cref="Value"/>.
		/// </summary>
		/// <param name="channel">MCMS channel.</param>
		/// <param name="propertyKey">Key of the property.</param>
		/// <param name="propertyValue">Value of the property.</param>
		/// <example>
		/// <code>
		/// public void CreateProperties(Channel channel)
		/// {
		///   CustomPropertyDbEx cp = new CustomPropertyDbEx(channel, "TimeZone", "EST");
		///   cp.Commit();
		/// }
		/// </code>
		/// </example>
		public CustomPropertyDbEx(Channel channel, string propertyKey, string propertyValue)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= CustomPropertyDbExAssetType.Channel;
			this.m_assetID		= new Guid(channel.Guid);
			this.m_cpKey		= propertyKey;
			this.m_cpValue		= propertyValue;
		}

		
		/// <summary>
		/// Creates a new <b>CustomPropertyDbEx</b> and sets the <see cref="AssetGuid"/> and 
		/// <see cref="AssetType"/> for the provided <paramref>template</paramref>.
		/// </summary>
		/// <param name="template">MCMS template.</param>
		/// <example>
		/// <code>
		/// public void CreateProperties(Template template)
		/// {
		///   CustomPropertyDbEx cp = new CustomPropertyDbEx(template);
		///   cp.Key = "TimeZone";
		///   cp.Value = "EST";
		///   cp.Commit();
		/// }
		/// </code>
		/// </example>
		public CustomPropertyDbEx(Template template)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= CustomPropertyDbExAssetType.Template;
			this.m_assetID		= new Guid(template.Guid);
		}

		
		/// <summary>
		/// Creates a new <b>CustomPropertyDbEx</b> and sets the <see cref="AssetGuid"/> and 
		/// <see cref="AssetType"/> for the provided <paramref>template</paramref> as well as 
		/// setting the <see cref="Key"/> and <see cref="Value"/>.
		/// </summary>
		/// <param name="template">MCMS template.</param>
		/// <param name="propertyKey">Key of the property.</param>
		/// <param name="propertyValue">Value of the property.</param>
		/// <example>
		/// <code>
		/// public void CreateProperties(Template template)
		/// {
		///   CustomPropertyDbEx cp = new CustomPropertyDbEx(template, "TimeZone", "EST");
		///   cp.Commit();
		/// }
		/// </code>
		/// </example>
		public CustomPropertyDbEx(Template template, string propertyKey, string propertyValue)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= CustomPropertyDbExAssetType.Template;
			this.m_assetID		= new Guid(template.Guid);
			this.m_cpKey		= propertyKey;
			this.m_cpValue		= propertyValue;
		}

		
		/// <summary>
		/// Creates a new <b>CustomPropertyDbEx</b> and sets the <see cref="AssetGuid"/> and 
		/// <see cref="AssetType"/> for the provided <paramref>resource</paramref>.
		/// </summary>
		/// <param name="resource">MCMS resource.</param>
		/// <example>
		/// <code>
		/// public void CreateProperties(Resource resource)
		/// {
		///   CustomPropertyDbEx cp = new CustomPropertyDbEx(resource);
		///   cp.Key = "TimeZone";
		///   cp.Value = "EST";
		///   cp.Commit();
		/// }
		/// </code>
		/// </example>
		public CustomPropertyDbEx(Resource resource)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= CustomPropertyDbExAssetType.Resource;
			this.m_assetID		= new Guid(resource.Guid);
		}

		
		/// <summary>
		/// Creates a new <b>CustomPropertyDbEx</b> and sets the <see cref="AssetGuid"/> and 
		/// <see cref="AssetType"/> for the provided <paramref>resource</paramref> as well as 
		/// setting the <see cref="Key"/> (with <paramref>propertyKey</paramref>) and 
		/// <see cref="Value"/> (with <paramref>propertyValue</paramref>).
		/// </summary>
		/// <param name="resource">MCMS resource.</param>
		/// <param name="propertyKey">Key of the property.</param>
		/// <param name="propertyValue">Value of the property.</param>
		/// <example>
		/// <code>
		/// public void CreateProperties(Resource resource)
		/// {
		///   CustomPropertyDbEx cp = new CustomPropertyDbEx(resource, "TimeZone", "EST");
		///   cp.Commit();
		/// }
		/// </code>
		/// </example>
		public CustomPropertyDbEx(Resource resource, string propertyKey, string propertyValue)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= CustomPropertyDbExAssetType.Resource;
			this.m_assetID		= new Guid(resource.Guid);
			this.m_cpKey		= propertyKey;
			this.m_cpValue		= propertyValue;
		}

		
		/// <summary>
		/// Creates a new <b>CustomPropertyDbEx</b> and sets the <see cref="AssetGuid"/> and 
		/// <see cref="AssetType"/> for the provided <paramref>posting</paramref>.
		/// </summary>
		/// <param name="posting">MCMS posting.</param>
		/// <example>
		/// <code>
		/// public void CreateProperties(Posting posting)
		/// {
		///   CustomPropertyDbEx cp = new CustomPropertyDbEx(posting);
		///   cp.Key = "TimeZone";
		///   cp.Value = "EST";
		///   cp.Commit();
		/// }
		/// </code>
		/// </example>
		public CustomPropertyDbEx(Posting posting)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= CustomPropertyDbExAssetType.Posting;
			this.m_assetID		= new Guid(posting.Guid);
		}


		/// <summary>
		/// Creates a new <b>CustomPropertyDbEx</b> and sets the <see cref="AssetGuid"/> and 
		/// <see cref="AssetType"/> for the provided <paramref>posting</paramref> as well as 
		/// setting the <see cref="Key"/> (with <paramref>propertyKey</paramref>) and 
		/// <see cref="Value"/> (with <paramref>propertyValue</paramref>).
		/// </summary>
		/// <param name="posting">MCMS posting.</param>
		/// <param name="propertyKey">Key of the property.</param>
		/// <param name="propertyValue">Value of the property.</param>
		/// <example>
		/// <code>
		/// public void CreateProperties(Posting posting)
		/// {
		///   CustomPropertyDbEx cp = new CustomPropertyDbEx(posting, "TimeZone", "EST");
		///   cp.Commit();
		/// }
		/// </code>
		/// </example>
		public CustomPropertyDbEx(Posting posting, string propertyKey, string propertyValue)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= CustomPropertyDbExAssetType.Posting;
			this.m_assetID		= new Guid(posting.Guid);
			this.m_cpKey		= propertyKey;
			this.m_cpValue		= propertyValue;
		}


		/// <summary>
		/// Creates a new <b>CustomPropertyDbEx</b> and sets the <see cref="AssetType"/> and 
		/// <see cref="AssetGuid"/> using the provided variables (<paramref>assetType</paramref> 
		/// and <paramref>assetGuid</paramref>).
		/// <seealso cref="ContentManagement.Extensions.CustomPropertyDbExAssetType"/>
		/// </summary>
		/// <param name="assetType">Type of the asset property is linked to.</param>
		/// <param name="assetGuid">Unique ID of the asset the the property is linked to.</param>
		/// <example>
		/// <code>
		/// public void CreateProperties(Channel channel)
		/// {
		///   CustomPropertyDbEx cp = new CustomPropertyDbEx(CustomPropertyDbExAssetType.Channel, new Guid(channel.Guid.ToString()));
		///   cp.Key = "TimeZone";
		///   cp.Value = "EST";
		///   cp.Commit();
		/// }
		/// </code>
		/// </example>
		public CustomPropertyDbEx(CustomPropertyDbExAssetType assetType, Guid assetGuid)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= assetType;
			this.m_assetID		= assetGuid;
		}


		/// <summary>
		/// Creates a new <b>CustomPropertyDbEx</b> and sets the <see cref="AssetType"/> and 
		/// <see cref="AssetGuid"/> using the provided variables (<paramref>assetType</paramref> 
		/// and <paramref>assetGuid</paramref>) as well as the <see cref="Key"/> (with 
		/// <paramref>propertyKey</paramref>) and <see cref="Value"/> (<paramref>propertyValue</paramref>).
		/// <seealso cref="ContentManagement.Extensions.CustomPropertyDbExAssetType"/>
		/// </summary>
		/// <param name="assetType">Type of the asset property is linked to.</param>
		/// <param name="assetGuid">Unique ID of the asset the the property is linked to.</param>
		/// <param name="propertyKey">Key of the property.</param>
		/// <param name="propertyValue">Value of the property.</param>
		/// <example>
		/// <code>
		/// public void CreateProperties(Channel channel)
		/// {
		///   CustomPropertyDbEx cp = new CustomPropertyDbEx(CustomPropertyDbExAssetType.Channel, new Guid(channel.Guid.ToString()), "TimeZone", "EST");
		///   cp.Commit();
		/// }
		/// </code>
		/// </example>
		public CustomPropertyDbEx(CustomPropertyDbExAssetType assetType, Guid assetGuid, string propertyKey, string propertyValue)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();
			
			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= assetType;
			this.m_assetID		= assetGuid;
			this.m_cpKey			= propertyKey;
			this.m_cpValue		= propertyValue;
		}


		/// <summary>
		/// Creates a new <b>CustomPropertyDbEx</b> and sets the <see cref="AssetType"/> and 
		/// <see cref="AssetGuid"/> using the provided variables (<paramref>assetType</paramref> 
		/// and <paramref>assetGuid</paramref>) as well as the <see cref="Key"/> (with 
		/// <paramref>propertyKey</paramref>) and <see cref="Value"/> (<paramref>propertyValue</paramref>).
		/// <seealso cref="ContentManagement.Extensions.CustomPropertyDbExAssetType"/>
		/// </summary>
		/// <remarks>
		/// Available internally only to create a new custom properties based on search results.
		/// </remarks>
		/// <param name="propertyGuid">Unique ID of the custom property.</param>
		/// <param name="assetType">Type of the asset property is linked to.</param>
		/// <param name="assetGuid">Unique ID of the asset the the property is linked to.</param>
		/// <param name="propertyKey">Key of the property.</param>
		/// <param name="propertyValue">Value of the property.</param>
		internal CustomPropertyDbEx(Guid propertyGuid, CustomPropertyDbExAssetType assetType, Guid assetGuid, string propertyKey, string propertyValue)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();
			
			// set flags
			this.m_deletePropertyFlag			= false;
			this.m_existingPropertyFlag		= true;

			// set values
			this.m_uniqueID		= propertyGuid;
			this.m_assetType	= assetType;
			this.m_assetID		= assetGuid;
			this.m_cpKey			= propertyKey;
			this.m_cpValue		= propertyValue;
		}

		#endregion

		#region Private/protected/internal methods
		#endregion

		#region Public methods
		/// <summary>
		/// Create an exact copy of the current <b>CustomPropertyDbEx</b>, except it replaces the 
		/// the <see cref="AssetType"/> and <see cref="AssetGuid"/> in the new property with the 
		/// provided CMS asset.
		/// </summary>
		/// <remarks>
		/// This method is commonly used for copying custom properties from one CMS asset to another.
		/// <b>Does not include <see cref="Commit"/>.</b>  You must call <see cref="Commit"/> to save 
		/// changes to the database.
		/// </remarks>
		/// <param name="channel">MCMS channel to assign the new property to.</param>
		/// <returns>Copy of the <b>CustomPropertyDbEx</b>, but with a specific asset type and asset id changed.</returns>
		/// <example>
		/// <code>
		/// public CustomPropertyDbEx CopyProperty(CustomPropertyDbEx origCP, Channel channel)
		/// {
		///   // copy create the new property
		///   CustomPropertyDbEx newCP = origCP.Clone(channel);
		///   // save the new property to the database
		///   newCP.Commit();
		///   // return the new property to the calling application
		///   return newCP;
		/// }
		/// </code>
		/// </example>
		public CustomPropertyDbEx Clone(Channel channel)
		{
			return new CustomPropertyDbEx(this.Guid, CustomPropertyDbExAssetType.Channel, new Guid(channel.Guid.ToString()), this.Key, this.Value);
		}


		/// <summary>
		/// Create an exact copy of the current <b>CustomPropertyDbEx</b>, except it replaces the 
		/// the <see cref="AssetType"/> and <see cref="AssetGuid"/> in the new property with the 
		/// provided CMS asset.
		/// </summary>
		/// <remarks>
		/// This method is commonly used for copying custom properties from one CMS asset to another.
		/// <b>Does not include <see cref="Commit"/>.</b>  You must call <see cref="Commit"/> to save 
		/// changes to the database.
		/// </remarks>
		/// <param name="template">MCMS template to assign the new property to.</param>
		/// <returns>Copy of the <b>CustomPropertyDbEx</b>, but with a specific asset type and asset id changed.</returns>
		/// <example>
		/// <code>
		/// public CustomPropertyDbEx CopyProperty(CustomPropertyDbEx origCP, Template template)
		/// {
		///   // copy create the new property
		///   CustomPropertyDbEx newCP = origCP.Clone(template);
		///   // save the new property to the database
		///   newCP.Commit();
		///   // return the new property to the calling application
		///   return newCP;
		/// }
		/// </code>
		/// </example>
		public CustomPropertyDbEx Clone(Template template)
		{
			return new CustomPropertyDbEx(this.Guid, CustomPropertyDbExAssetType.Template, new Guid(template.Guid.ToString()), this.Key, this.Value);
		}


		/// <summary>
		/// Create an exact copy of the current <b>CustomPropertyDbEx</b>, except it replaces the 
		/// the <see cref="AssetType"/> and <see cref="AssetGuid"/> in the new property with the 
		/// provided CMS asset.
		/// </summary>
		/// <remarks>
		/// This method is commonly used for copying custom properties from one CMS asset to another.
		/// <b>Does not include <see cref="Commit"/>.</b>  You must call <see cref="Commit"/> to save 
		/// changes to the database.
		/// </remarks>
		/// <param name="resource">MCMS resource to assign the new property to.</param>
		/// <returns>Copy of the <b>CustomPropertyDbEx</b>, but with a specific asset type and asset id changed.</returns>
		/// <example>
		/// <code>
		/// public CustomPropertyDbEx CopyProperty(CustomPropertyDbEx origCP, Resource resource)
		/// {
		///   // copy create the new property
		///   CustomPropertyDbEx newCP = origCP.Clone(resource);
		///   // save the new property to the database
		///   newCP.Commit();
		///   // return the new property to the calling application
		///   return newCP;
		/// }
		/// </code>
		/// </example>
		public CustomPropertyDbEx Clone(Resource resource)
		{
			return new CustomPropertyDbEx(this.Guid, CustomPropertyDbExAssetType.Resource, new Guid(resource.Guid.ToString()), this.Key, this.Value);
		}


		/// <summary>
		/// Create an exact copy of the current <b>CustomPropertyDbEx</b>, except it replaces the 
		/// the <see cref="AssetType"/> and <see cref="AssetGuid"/> in the new property with the 
		/// provided CMS asset.
		/// </summary>
		/// <remarks>
		/// This method is commonly used for copying custom properties from one CMS asset to another.
		/// <b>Does not include <see cref="Commit"/>.</b>  You must call <see cref="Commit"/> to save 
		/// changes to the database.
		/// </remarks>
		/// <param name="posting">MCMS posting to assign the new property to.</param>
		/// <returns>Copy of the <b>CustomPropertyDbEx</b>, but with a specific asset type and asset id changed.</returns>
		/// <example>
		/// <code>
		/// public CustomPropertyDbEx CopyProperty(CustomPropertyDbEx origCP, Posting posting)
		/// {
		///   // copy create the new property
		///   CustomPropertyDbEx newCP = origCP.Clone(posting);
		///   // save the new property to the database
		///   newCP.Commit();
		///   // return the new property to the calling application
		///   return newCP;
		/// }
		/// </code>
		/// </example>
		public CustomPropertyDbEx Clone(Posting posting)
		{
			return new CustomPropertyDbEx(this.Guid, CustomPropertyDbExAssetType.Posting, new Guid(posting.Guid.ToString()), this.Key, this.Value);
		}


		/// <summary>
		/// Create an exact copy of the current <b>CustomPropertyDbEx</b>, except it replaces the 
		/// the <see cref="AssetType"/> and <see cref="AssetGuid"/> in the new property.
		/// </summary>
		/// <remarks>
		/// This method is commonly used for copying custom properties from one CMS asset to another.
		/// <b>Does not include <see cref="Commit"/>.</b>  You must call <see cref="Commit"/> to save 
		/// changes to the database.
		/// </remarks>
		/// <param name="newAssetType">Type of asset for new property.</param>
		/// <param name="newAssetGuid">Unique ID of asset for new property.</param>
		/// <returns>Copy of the <b>CustomPropertyDbEx</b>, but with a specific asset type and asset id changed.</returns>
		/// <example>
		/// <code>
		/// public CustomPropertyDbEx CopyProperty(CustomPropertyDbEx origCP, Channel targetChannel)
		/// {
		///   // copy create the new property
		///   CustomPropertyDbEx newCP = origCP.Clone(CustomPropertyDbExAssetType.Channel, new Guid(targetChannel.Guid.ToString()));
		///   // save the new property to the database
		///   newCP.Commit();
		///   // return the new property to the calling application
		///   return newCP;
		/// }
		/// </code>
		/// </example>
		public CustomPropertyDbEx Clone(CustomPropertyDbExAssetType newAssetType, Guid newAssetGuid)
		{
			return new CustomPropertyDbEx(this.Guid, newAssetType, newAssetGuid, this.Key, this.Value);
		}


		/// <summary>
		/// Refreshes the current <b>CustomPropertyDbEx</b> with the latest information from the database.
		/// </summary>
		/// <remarks>
		/// If any changes to this <b>CustomPropertyDbEx</b> have been made before calling 
		/// <see cref="Commit"/>, they will be lost.
		/// </remarks>
		/// <example>
		/// <code>
		/// public CustomPropertyDbEx RefreshProperty(CustomPropertyDbEx cp)
		/// {
		///   cp.Sync();
		///   return cp;
		/// }
		/// </code>
		/// </example>
		public void Sync()
		{
			// make sure there's an object
			if (this.m_uniqueID.ToString().Length == 0)
				throw new Exception("No custom property currently loaded to sync.");


			// get property
			DataRow dr = this.m_dataHandler.Select_ByID(this.m_uniqueID);


			// refresh property values
			switch (dr[CustomPropertyTableField.ID.ToString()].ToString())
			{
				case "Site":
					this.m_assetType = CustomPropertyDbExAssetType.Site;
					break;
				case "Channel":
					this.m_assetType = CustomPropertyDbExAssetType.Channel;
					break;
				case "Template":
					this.m_assetType = CustomPropertyDbExAssetType.Template;
					break;
				case "Resource":
					this.m_assetType = CustomPropertyDbExAssetType.Resource;
					break;
				case "Posting":
					this.m_assetType = CustomPropertyDbExAssetType.Posting;
					break;
				case "Other":
					this.m_assetType = CustomPropertyDbExAssetType.Other;
					break;
			}
			this.m_assetID		= new Guid(dr[CustomPropertyTableField.AssetID.ToString()].ToString());
			this.m_cpKey			= dr[CustomPropertyTableField.PropertyKey.ToString()].ToString();
			this.m_cpValue		= dr[CustomPropertyTableField.PropertyValue.ToString()].ToString();


			// update flags
			this.m_existingPropertyFlag = false;
		}


		/// <summary>
		/// Saves all current settings of the current <b>CustomPropertyDbEx</b> to the database.
		/// </summary>
		public void Commit()
		{
			// make sure there's an object
			if (this.m_uniqueID.ToString().Length == 0)
				throw new Exception("No custom property currently loaded to commit.");


			// if delete..
			if (this.m_deletePropertyFlag)
			{
				#region delete current property
				this.m_dataHandler.DeleteProperty(this.m_uniqueID);

				// reset all internal objects
				// TODO: need to null-out the entire object
				this.m_existingPropertyFlag = false;
				this.m_deletePropertyFlag = false;
				this.m_cpKey = null;
				this.m_cpValue = null;
				#endregion
			}
				// else, if it already existed, update it...
			else if (this.m_existingPropertyFlag)
			{
				#region update existing current property
				this.m_dataHandler.Update(this.m_uniqueID, this.m_assetType.ToString(), this.m_assetID, this.m_cpKey, this.m_cpValue);
				#endregion
			}
				// else, create new one
			else
			{
				#region create new custom property
				this.m_dataHandler.Insert(this.m_uniqueID, this.m_assetType.ToString(), this.m_assetID, this.m_cpKey, this.m_cpValue);

				this.m_existingPropertyFlag = true;
				#endregion
			}
		}


		/// <summary>
		/// Removes the current <b>CustomPropertyDbEx</b> from the system.
		/// </summary>
		/// <remarks>
		/// Must call <see cref="Commit"/> to take effect database.  Calling <b>Delete</b> only
		/// flags the property for deletion.
		/// </remarks>
		public void Delete()
		{
			this.m_deletePropertyFlag = true;
		}
		#endregion

		#region Public properties
		/// <summary>
		/// Unique identifier.
		/// <seealso cref="System.Guid"/>
		/// </summary>
		public Guid Guid
		{
			get {return this.m_uniqueID;}
		}


		/// <summary>
		/// Type of CMS asset/object the <b>CustomPropertyDbEx</b> is assigned to.
		/// <seealso cref="ContentManagement.Extensions.CustomPropertyDbExAssetType"/>
		/// </summary>
		public CustomPropertyDbExAssetType AssetType
		{
			get{return this.m_assetType;}
			set{this.m_assetType = value;}
		}


		/// <summary>
		/// Unique ID of the CMS asset this <b>CustomPropertyDbEx</b> belongs to.
		/// </summary>
		public Guid AssetGuid
		{
			get{return this.m_assetID;}
			set{this.m_assetID = value;}
		}


		/// <summary>
		/// Property key.
		/// </summary>
		public string Key
		{
			get{return this.m_cpKey;}
			set{this.m_cpKey = value;}
		}


		/// <summary>
		/// Property value.
		/// </summary>
		public string Value
		{
			get{return this.m_cpValue;}
			set{this.m_cpValue = value;}
		}
		#endregion

		#region Event Handlers
		#endregion

		#region Comparison objects
		#endregion	
	}
}