#region Namespace references
/// Microsoft .NET Framework namespaces
using System;
using System.Data;
/// Microsoft CMS 2002 namespaces
using Microsoft.ContentManagement.Publishing;
#endregion

namespace ContentManagement.Extensions
{
	/// <summary>
	/// Class of static methods and objects used for searching and managing the <b>CustomPropertyDbEx</b>
	/// objects and the underlying database.
	/// </summary>
	public class CustomPropertyDbExGlobal
	{
		#region Constructor & Descructor
		/// <summary>
		/// Prohibits the initialization of a new instance of the <b>CustomPropertyDbExGlobal</b> class.
		/// </summary>
		/// <remarks>
		/// All properties and methods will be static and won't need to be initialized.
		/// </remarks>
		private CustomPropertyDbExGlobal(){}
		#endregion

		#region SUBCLASS Searches
		/// <summary>
		/// Class of static methods used for searching for <b>CustomPropertyDbEx</b> in the database.
		/// </summary>
		public class Searches
		{
			#region Constants
			#endregion

			#region Objects
			// data access layer object
			private static DataHandler		m_dataHandler;
			#endregion

			#region Constructor & Descructor
			/// <summary>
			/// Prohibits the initialization of a new instance of the <b>Searches</b> class.
			/// </summary>
			/// <remarks>
			/// All properties and methods will be static and won't need to be initialized.
			/// </remarks>
			private Searches(){}
			#endregion

			#region Private/protected/internal methods
			/// <summary>
			/// Loads the <see cref="DataHandler"/> if it's not already initalized.
			/// </summary>
			/// <returns>Initalized <see cref="DataHandler"/>.</returns>
			private static DataHandler GetDataHandler()
			{
				if (m_dataHandler == null)
					m_dataHandler = new DataHandler();

				return m_dataHandler;
			}
			

			/// <summary>
			/// Finds the custom property(s) matching the specific asset's guid.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="assetGuid"><see cref="System.Guid"/> unique ID of the desired <parmref>assetGuid</parmref>.</param>
			/// <returns><see cref="CustomPropertyDbExCollection"/> containing matching the specified <parmref>assetGuid</parmref>.</returns>
			internal static CustomPropertyDbExCollection GetByAssetGuid(Guid assetGuid)
			{
				DataSet ds = GetDataHandler().Select_ByAssetID(assetGuid);

				if (ds != null)
					return Transform.ToCustomPropertyDbEx(ds);
				else
					return null;
			}
			

			/// <summary>
			/// Finds the custom property(s) matching the specific asset guid & key.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="assetGuid"><see cref="System.Guid"/> unique ID of the desired <parmref>assetGuid</parmref>.</param>
			/// <param name="key">Property key of property desired.</param>
			/// <returns><see cref="CustomPropertyDbExCollection"/> containing matching the found property.</returns>
			internal static CustomPropertyDbExCollection GetByKeyForAsset(Guid assetGuid, string key)
			{
				DataSet ds = GetDataHandler().Select_ByAssetID(assetGuid);

				// if properties found...
				if (ds != null)
				{
					// get all properties with specific key for this asset
					DataRow[] dr = ds.Tables[0].Select(
						string.Format("{0}='{1}'", CustomPropertyTableField.PropertyKey.ToString(), key), 
						string.Format("{0} ASC", CustomPropertyTableField.PropertyValue.ToString()), 
						DataViewRowState.OriginalRows);
				
					// if properties with this key found...
					if (dr.Length == 0)
						return null;
					else
					{
						CustomPropertyDbExCollection collection = new CustomPropertyDbExCollection();
						foreach(DataRow r in dr)
							collection.Add(Transform.ToCustomPropertyDbEx(r));

						return collection;
					}
				}
				else
					return null;
			}
			#endregion

			#region Public methods
			/// <summary>
			/// Retrives all custom properties for the specified <paramref>channel</paramref>.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="channel">MCMS Channel object containing custom properties.</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties for the <paramref>channel</paramref>.</returns>
			/// <example>
			/// <code>
			/// CustomPropertyDbExCollection collection = CustomPropertyDbExGlobal.Searches.GetProperties(channel);
			/// </code>
			/// </example>
			public static CustomPropertyDbExCollection GetProperties(Channel channel)
			{
				return GetByAssetGuid(new Guid(channel.Guid.ToString()));
			}


			/// <summary>
			/// Retrives all custom properties for the specified channel and property key.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="channel">MCMS Channel object containing custom properties.</param>
			/// <param name="key">Property key of property desired.</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties for the <paramref>channel</paramref>.</returns>
			/// <example>
			/// <code>
			/// CustomPropertyDbExCollection collection = CustomPropertyDbExGlobal.Searches.GetProperties(channel, propertyKey);
			/// </code>
			/// </example>
			public static CustomPropertyDbExCollection GetProperties(Channel channel, string key)
			{
				return GetByKeyForAsset(new Guid(channel.Guid.ToString()), key);
			}


			/// <summary>
			/// Retrives all custom properties for the specified template.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="template">MCMS Template object containing custom properties.</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties for the <paramref>template</paramref>.</returns>
			/// <example>
			/// <code>
			/// CustomPropertyDbExCollection collection = CustomPropertyDbExGlobal.Searches.GetProperties(template);
			/// </code>
			/// </example>
			public static CustomPropertyDbExCollection GetProperties(Template template)
			{
				return GetByAssetGuid(new Guid(template.Guid.ToString()));
			}


			/// <summary>
			/// Retrives all custom properties for the specified template and property key.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="template">MCMS Template object containing custom properties.</param>
			/// <param name="key">Property key of property desired.</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties for the <paramref>template</paramref>.</returns>
			/// <example>
			/// <code>
			/// CustomPropertyDbExCollection collection = CustomPropertyDbExGlobal.Searches.GetProperties(template, propertyKey);
			/// </code>
			/// </example>
			public static CustomPropertyDbExCollection GetProperties(Template template, string key)
			{
				return GetByKeyForAsset(new Guid(template.Guid.ToString()), key);
			}


			/// <summary>
			/// Retrives all custom properties for the specified posting.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="posting">MCMS Posting object containing custom properties.</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties for the <paramref>posting</paramref>.</returns>
			/// <example>
			/// <code>
			/// CustomPropertyDbExCollection collection = CustomPropertyDbExGlobal.Searches.GetProperties(posting);
			/// </code>
			/// </example>
			public static CustomPropertyDbExCollection GetProperties(Posting posting)
			{
				return GetByAssetGuid(new Guid(posting.Guid.ToString()));
			}
	

			/// <summary>
			/// Retrives all custom properties for the specified posting and property key.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="posting">MCMS Posting object containing custom properties.</param>
			/// <param name="key">Property key of property desired.</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties for the <paramref>posting</paramref>.</returns>
			/// <example>
			/// <code>
			/// CustomPropertyDbExCollection collection = CustomPropertyDbExGlobal.Searches.GetProperties(posting, propertyKey);
			/// </code>
			/// </example>
			public static CustomPropertyDbExCollection GetProperties(Posting posting, string key)
			{
				return GetByKeyForAsset(new Guid(posting.Guid.ToString()), key);
			}
	

			/// <summary>
			/// Retrives all custom properties for the specified resource.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="resource">MCMS Resource object containing custom properties.</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties for the <paramref>resource</paramref>.</returns>
			/// <example>
			/// <code>
			/// CustomPropertyDbExCollection collection = CustomPropertyDbExGlobal.Searches.GetProperties(resource);
			/// </code>
			/// </example>
			public static CustomPropertyDbExCollection GetProperties(Resource resource)
			{
				return GetByAssetGuid(new Guid(resource.Guid.ToString()));
			}


			/// <summary>
			/// Retrives all custom properties for the specified resource and property key.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="resource">MCMS Resource object containing custom properties.</param>
			/// <param name="key">Property key of property desired.</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties for the <paramref>resource</paramref>.</returns>
			/// <example>
			/// <code>
			/// CustomPropertyDbExCollection collection = CustomPropertyDbExGlobal.Searches.GetProperties(resource, propertyKey);
			/// </code>
			/// </example>
			public static CustomPropertyDbExCollection GetProperties(Resource resource, string key)
			{
				return GetByKeyForAsset(new Guid(resource.Guid.ToString()), key);
			}


			/// <summary>
			/// Finds the custom property matching a specific guid.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="propertyGuid"><see cref="System.Guid"/> unique ID of the desired property.</param>
			/// <returns><see cref="CustomPropertyDbEx"/> matching the found <paramref>propertyGuid</paramref>.</returns>
			/// <example>
			/// <code>
			/// System.Guid guid = new System.Guid("30C5AEB7-87D7-4FFB-887C-01243818DD38");
			/// CustomPropertyDbExCollection collection = CustomPropertyDbExGlobal.Searches.GetByGuid(guid);
			/// </code>
			/// </example>
			public static CustomPropertyDbEx GetByGuid(Guid propertyGuid)
			{
				DataRow r = GetDataHandler().Select_ByID(propertyGuid);
				if (r != null)
					return Transform.ToCustomPropertyDbEx(r);
				else
					return null;
			}


			/// <summary>
			/// Finds the custom property(s) matching a specific asset type.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="assetType"><see cref="CustomPropertyDbExAssetType" /> of the desired property(s).</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties matching the found <paramref>assetType</paramref>.</returns>
			/// <example>
			/// <code>
			/// CustomPropertyDbExCollection collection = CustomPropertyDbExGlobal.Searches.GetByAssetType(CustomPropertyDbExAssetType.Channel);
			/// </code>
			/// </example>
			public static CustomPropertyDbExCollection GetByAssetType(CustomPropertyDbExAssetType assetType)
			{
				DataSet ds = GetDataHandler().Select(assetType.ToString(), null, null);
				if (ds != null)
					return Transform.ToCustomPropertyDbEx(ds);
				else
					return null;
			}



			/// <summary>
			/// Finds the custom property(s) matching the specific property value.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="value">Value of the desired property(s).</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties matching the found property's value.</returns>
			/// <example>
			/// <code>
			/// string propertyValue = "admin%";
			/// CustomPropertyDbExCollection collection = CustomPropertyDbExGlobal.Searches.GetByValue(propertyValue);
			/// </code>
			/// </example>
			public static CustomPropertyDbExCollection GetByValue(string value)
			{
				DataSet ds = GetDataHandler().Select_ByValue(value);

				if (ds != null)
					return Transform.ToCustomPropertyDbEx(ds);
				else
					return null;
			}


			/// <summary>
			/// Finds the custom property(s) matching the specific property key.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="value">Key of the desired property(s).</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties matching the found property's key.</returns>
			/// <example>
			/// <code>
			/// string propertyKey = "admin%";
			/// CustomPropertyDbExCollection collection = CustomPropertyDbExGlobal.Searches.GetByKey(propertyKey);
			/// </code>
			/// </example>
			public static CustomPropertyDbExCollection GetByKey(string value)
			{
				DataSet ds = GetDataHandler().Select_ByKey(value);

				if (ds != null)
					return Transform.ToCustomPropertyDbEx(ds);
				else
					return null;
			}
			#endregion
		}
		#endregion
	}
}