#region Namespace references
/// Microsoft .NET Framework namespaces
using System;
using System.Data;
using System.Data.SqlClient;
/// Microsoft data application block
using Microsoft.ApplicationBlocks.Data;
#endregion

namespace ContentManagement.Extensions
{
	/// <summary>
	/// Summary description for DataHandler.
	/// </summary>
	internal class DataHandler
	{
		#region Constants
		private const string c_storedProcedurePrefix = "cpdbex_";
		#endregion

		#region Objects
		private string m_connectionString = string.Empty;
		#endregion

		#region Constructor & Descructor
		/// <summary>
		/// Class constructor
		/// </summary>
		internal DataHandler()
		{
			this.InitConnectionString(null);
		}
		#endregion

		#region Private/protected/internal methods
		/// <summary>
		/// Inits the connection string.
		/// </summary>
		/// <param name="connectionString">Database connection string.</param>
		private void InitConnectionString(string connectionString)
		{
			if (connectionString != null)
				this.m_connectionString = connectionString;
			else
				this.m_connectionString = System.Configuration.ConfigurationSettings.AppSettings["ContentManagement.CustomPropertyDbEx.ConnectionString"].ToString();

			// if no connection string, die.
			if (this.m_connectionString == null)
				throw new Exception("No connection string found.");
		}
		#endregion

		#region Public methods
		/// <summary>
		/// Creates a new property ID record
		/// </summary>
		/// <param name="propertyID">Property guid.</param>
		/// <param name="assetType">Asset type.</param>
		/// <param name="assetID">Asset guid.</param>
		/// <param name="propertyKey">Property key.</param>
		/// <param name="propertyValue">Property value.</param>
		internal void Insert(Guid propertyID, string assetType, Guid assetID, string propertyKey, string propertyValue)
		{
			string sp = "[dbo].[" +c_storedProcedurePrefix +"insCustomProperty]";
			
			// create parm(s)
			SqlParameter[] parms =
			{
				SqlUtility.MakeInParam("@ID",							propertyID),
				SqlUtility.MakeInParam("@assetType",			assetType),
				SqlUtility.MakeInParam("@assetID",				assetID),
				SqlUtility.MakeInParam("@propertyKey",		propertyKey),
				SqlUtility.MakeInParam("@propertyValue",	DataHelper.CheckNullString(propertyValue))
			};

			// execute the statement
			try
			{SqlHelper.ExecuteNonQuery(this.m_connectionString, CommandType.StoredProcedure, sp, parms);}
			catch(Exception ex)
			{throw ex;}
		}


		/// <summary>
		/// Updates existing property.
		/// </summary>
		/// <param name="propertyID">Property guid.</param>
		/// <param name="assetType">Asset type.</param>
		/// <param name="assetID">Asset guid.</param>
		/// <param name="propertyKey">Property key.</param>
		/// <param name="propertyValue">Property value.</param>
		internal void Update(Guid propertyID, string assetType, Guid assetID, string propertyKey, string propertyValue)
		{
			string sp = "[dbo].[" +c_storedProcedurePrefix +"updCustomProperty]";
			
			// create parm(s)
			SqlParameter[] parms =
			{
				SqlUtility.MakeInParam("@ID",							propertyID),
				SqlUtility.MakeInParam("@assetType",			assetType),
				SqlUtility.MakeInParam("@assetID",				assetID),
				SqlUtility.MakeInParam("@propertyKey",		propertyKey),
				SqlUtility.MakeInParam("@propertyValue",	DataHelper.CheckNullString(propertyValue))
			};

			// execute the statement
			try
			{SqlHelper.ExecuteNonQuery(this.m_connectionString, CommandType.StoredProcedure, sp, parms);}
			catch(Exception ex)
			{throw ex;}
		}


		/// <summary>
		/// Removes specified property.
		/// </summary>
		/// <param name="propertyID">Property guid.</param>
		internal void DeleteProperty(Guid propertyID)
		{
			string sp = "[dbo].[" +c_storedProcedurePrefix +"delCustomProperty_byID]";
			
			// create parm(s)
			SqlParameter[] parms =
			{
				SqlUtility.MakeInParam("@ID",							propertyID)
			};

			// execute the statement
			try
			{SqlHelper.ExecuteNonQuery(this.m_connectionString, CommandType.StoredProcedure, sp, parms);}
			catch(Exception ex)
			{throw ex;}
		}


		/// <summary>
		/// Removes all properties for the specified asset.
		/// </summary>
		/// <param name="assetID">Asset guid.</param>
		internal void DeleteAssetProperties(Guid assetID)
		{
			string sp = "[dbo].[" +c_storedProcedurePrefix +"delCustomProperty_byID]";
			
			// create parm(s)
			SqlParameter[] parms =
			{
				SqlUtility.MakeInParam("@assetID", assetID)
			};

			// execute the statement
			try
			{SqlHelper.ExecuteNonQuery(this.m_connectionString, CommandType.StoredProcedure, sp, parms);}
			catch(Exception ex)
			{throw ex;}
		}


		/// <summary>
		/// Select property by guid.
		/// </summary>
		/// <param name="id">Property guid.</param>
		internal DataRow Select_ByID(Guid id)
		{
			string sp = "[dbo].[" +c_storedProcedurePrefix +"selCustomProperty_byID]";

			// create parm(s)
			SqlParameter[] parms =
			{
				SqlUtility.MakeInParam("@ID", id)
			};

			try
			{
				// execute the statement
				DataSet ds = SqlHelper.ExecuteDataset(m_connectionString, CommandType.StoredProcedure, sp, parms);
				return ds.Tables[0].Rows[0];
			}
			catch(Exception ex)
			{throw ex;}
		}


		/// <summary>
		/// Select properties by asset guid.
		/// </summary>
		/// <param name="assetID">Asset guid.</param>
		internal DataSet Select_ByAssetID(Guid assetID)
		{
			string sp = "[dbo].[" +c_storedProcedurePrefix +"selCustomProperty_byAssetID]";

			// create parm(s)
			SqlParameter[] parms =
			{
				SqlUtility.MakeInParam("@ID", assetID)
			};

			try
			{
				// execute the statement
				DataSet ds = SqlHelper.ExecuteDataset(m_connectionString, CommandType.StoredProcedure, sp, parms);
				return ds;
			}
			catch(Exception ex)
			{throw ex;}
		}


		/// <summary>
		/// Select properties by various properties.
		/// </summary>
		/// <remarks>
		/// Can use SQL wildcards in search.
		/// </remarks>
		/// <param name="assetType">Asset type.</param>
		/// <param name="propertyKey">Property key.</param>
		/// <param name="propertyValue">Property value.</param>
		/// <returns></returns>
		internal DataSet Select(string assetType, string propertyKey, string propertyValue)
		{
			string sp = "[dbo].[" +c_storedProcedurePrefix +"selCustomProperty_byAtPkPv]";

			// if any parms are null, set them to wildcards
			if (assetType == null || assetType == string.Empty)
				assetType = "%";
			if (propertyKey == null || propertyKey == string.Empty)
				propertyKey = "%";
			if (propertyValue == null || propertyValue == string.Empty)
				propertyValue = "%";

			// create parm(s)
			SqlParameter[] parms =
			{
				SqlUtility.MakeInParam("@assetType", assetType),
				SqlUtility.MakeInParam("@propertyKey", propertyKey),
				SqlUtility.MakeInParam("@propertyValue", propertyValue)
			};

			try
			{
				// execute the statement
				DataSet ds = SqlHelper.ExecuteDataset(m_connectionString, CommandType.StoredProcedure, sp, parms);
				return ds;
			}
			catch(Exception ex)
			{throw ex;}
		}
	

		/// <summary>
		/// Select properties by property value.
		/// </summary>
		/// <remarks>
		/// Can use SQL wildcards in search.
		/// </remarks>
		/// <param name="propertyValue">Property value.</param>
		/// <returns></returns>
		internal DataSet Select_ByValue(string propertyValue)
		{
			string sp = "[dbo].[" +c_storedProcedurePrefix +"selCustomProperty_byPropertyValue]";

			// create parm(s)
			SqlParameter[] parms =
			{
				SqlUtility.MakeInParam("@propertyValue", propertyValue)
			};

			try
			{
				// execute the statement
				DataSet ds = SqlHelper.ExecuteDataset(m_connectionString, CommandType.StoredProcedure, sp, parms);
				return ds;
			}
			catch(Exception ex)
			{throw ex;}		
		}
			

		/// <summary>
		/// Select properties by property key.
		/// </summary>
		/// <remarks>
		/// Can use SQL wildcards in search.
		/// </remarks>
		/// <param name="propertyKey">Property key.</param>
		/// <returns></returns>
		internal DataSet Select_ByKey(string propertyValue)
		{
			string sp = "[dbo].[" +c_storedProcedurePrefix +"selCustomProperty_byPropertyKey]";

			// create parm(s)
			SqlParameter[] parms =
			{
				SqlUtility.MakeInParam("@propertyKey", propertyValue)
			};

			try
			{
				// execute the statement
				DataSet ds = SqlHelper.ExecuteDataset(m_connectionString, CommandType.StoredProcedure, sp, parms);
				return ds;
			}
			catch(Exception ex)
			{throw ex;}		
		}
		#endregion

		#region Public properties
		/// <summary>
		/// Database connection string.
		/// </summary>
		internal string ConnectionString
		{
			get{return this.m_connectionString;}
			set{this.m_connectionString = value;}
		}
		#endregion

		#region Event Handlers
		#endregion
	}
}